/* Color.cpp defines the non-trivial Color operations.
 * ...
 */
 
#include <string>               // string
#include <cctype>               // islower(), toupper()
#include "Color.h"
using namespace std;

// ------- Wavelength of a Color ----------------------------------

double wavelength(Color aColor)
{
   switch(aColor)
   {
      case RED:
                   return 6.5E-7;
      case ORANGE:
                   return 6.0E-7;
      case YELLOW:
                   return 5.8E-7;
      case GREEN: 
                   return 5.2E-7;
      case BLUE:
                   return 4.7E-7;
      case INDIGO:
                   return 4.4E-7;
      case VIOLET:
                   return 4.1E-7;
      default:
                   cerr << "\n*** Wavelength: invalid color received!\n";
                   return 0.0;
   }
}

// ------- Color Input --------------------------------------------
istream& operator>>(istream& in, Color& aColor)
{
   string colorString;
   in >> colorString;

   for (int i = 0; i < colorString.size(); i++)
      if (islower(colorString[i]))
         colorString[i] = toupper(colorString[i]);
   if (colorString == "RED")
      aColor = RED;
   else if (colorString == "ORANGE")
      aColor = ORANGE;
   else if (colorString == "YELLOW")
      aColor = YELLOW;
   else if (colorString == "GREEN")
      aColor = GREEN;
   else if (colorString == "BLUE")
      aColor = BLUE;
   else if (colorString == "INDIGO")
      aColor = INDIGO;
   else if (colorString == "VIOLET")
      aColor = VIOLET;
   else
      in.setstate(ios::failbit);

   return in;
}

// ------- Color Output -------------------------------------------

ostream& operator<<(ostream& out, Color aColor)
{
   switch(aColor)
   {
      case RED:
                   out << "RED";
      case ORANGE:
                   out << "ORANGE";
      case YELLOW:
                   out << "YELLOW";
      case GREEN:
                   out << "GREEN";
      case BLUE:
                   out << "BLUE";
      case INDIGO:
                   out << "INDIGO";
      case VIOLET:
                   out << "VIOLET";
      default:
                   cerr << "\n*** operator<<: invalid color received!\n";
   }
   return out;
}

// ------- Color Successor ----------------------------------------

Color next(Color aColor)
{    
   switch (aColor)
   {
      case RED:
                   return ORANGE;
      case ORANGE:
                   return YELLOW;
      case YELLOW:
                   return GREEN;
      case GREEN:
                   return BLUE;
      case BLUE:
                   return INDIGO;
      case INDIGO:
                   return VIOLET;
      case VIOLET:
                   return COLOR_OVERFLOW;
      default:
                   cerr << "\n*** next(): invalid color received!\n";
                   return COLOR_OVERFLOW;
   }
}
